function addThread(thread, not_override)
	-- Global initialization
	if not global.booktorio then
		global.booktorio = {}
	end
	if not global.booktorio.threads then
		global.booktorio.threads = {}
	end
	-- Verify conditions
	local internal_thread_name = getFormattedCaption(thread.name)
	local should_override      = not_override ~= true
	local should_update        = internal_thread_name and global.booktorio.threads[internal_thread_name] and (global.booktorio.threads[internal_thread_name].specified_version < thread.specified_version)
	
	if internal_thread_name and (not global.booktorio.threads[internal_thread_name] or should_override or should_update) then
		local topics_names  = {}
		local thread_topics = {}
		local topics_align  = {}
		for i, topic_def in pairs(thread.topics) do
			topics_names[i]  = getFormattedCaption(topic_def.name, topic_def.localized or true)
			thread_topics[i] = table.deepcopy(topic_def.topic)
			topics_align[i] = topic_def.align or "left"
		end
		
		global.booktorio.threads[internal_thread_name] = 
		{
			["localized"]         = thread.localized or true,
			["topics_names"]      = topics_names, 
			["topics"]            = thread_topics,
			["topics_align"]      = topics_align,
			["specified_version"] = thread.specified_version or 0
		}
	end
end

if not remote.interfaces["Booktorio"] then
	remote.add_interface("Booktorio",
	{
		add_thread = 
		function(thread, not_override)
			if type(thread) ~= "table" then 
				error("Value for 'add_thread' must be a table.")
			end
			addThread(thread, not_override)
		end,
		override_thread = 
		function(thread)
			if type(thread) ~= "table" then 
				error("Value for 'override_thread' must be a table.")
			end
			addThread(thread, false)
		end
	})
end

return {}