require("__Booktorio__/control-scripts/control-lib/control-lib-initialization")
require("__core__/lualib/mod-gui")

-- Global prefix
local w_prefix = "bk-wiki-"

function getFormattedCaption(text, localized)
	if text then
		local caption = nil
		if type(text) == "table" and next(text) then
			_, caption = next(text)
		else
			caption = text
		end
		if localized == true then
			return {caption}
		else
			return caption
		end	
	end
	return nil
end

function getDescrptionGUIPane(event, tab_name)
	local wiki = booktorio.gui.getElementByName(event.player_index, w_prefix.."main-frame")
	if wiki and wiki.valid then
		local wiki_info_pane =  booktorio.gui.getElementByName(event.player_index, w_prefix..tab_name.."info-pane")
		if wiki_info_pane and wiki_info_pane.valid then
			wiki_info_pane.clear()
			return wiki_info_pane
		end
	end
	return nil
end

function addTitleToDescrptionGUIPane(pane, tab_name, title_name, localized, horizontal_align)
	if pane and tab_name and title_name then
		if localized == nil then
			localized = true
		end
		local display_width = getScaledDisplayWidth(pane.player_index)
		-- Title flow
		local title_flow = booktorio.gui.addFlow(pane, 
		{
			name      = w_prefix..tab_name.."title-flow",
			direction = "vertical",
			style     = "bk-title-flow"
		})
		title_flow.style.width = display_width * 0.82
		title_flow.style.horizontal_align = horizontal_align
		-- Title
		booktorio.gui.addLabel(title_flow, 
		{
			name    = w_prefix..tab_name.."description-title", 
			caption = getFormattedCaption(title_name, localized),
			style   = "bk-title"
		})
		-- Separator line
		local line = booktorio.gui.addLine(title_flow, 
		{
			name    = w_prefix..tab_name.."title-line",
			style   = "bk-title-line"
		})
		line.style.width = display_width * 0.82
	end
end

function addSubtitleToDescrptionGUIPane(pane, tab_name, subtitle_name, localized, index, horizontal_align)
	if pane and tab_name and subtitle_name then
		if localized == nil then
			localized = true
		end
		local display_width = getScaledDisplayWidth(pane.player_index)
		-- Subtitle flow
		local subtitle_flow = booktorio.gui.addFlow(pane, 
		{
			name      = w_prefix..tab_name.."subtitle-flow-" .. index,
			direction = "vertical",
			style     = "bk-title-flow"
		})
		subtitle_flow.style.width = display_width * 0.41
		subtitle_flow.style.horizontal_align = horizontal_align
		-- Subtitle
		booktorio.gui.addLabel(subtitle_flow, 
		{
			name    = w_prefix..tab_name.."subtitle-" .. index, 
			caption = getFormattedCaption(subtitle_name, localized),
			style   = "bk-subtitle"
		})
		-- Separator line
		local line = booktorio.gui.addLine(subtitle_flow, 
		{
			name    = w_prefix..tab_name.."subtitle-line-" .. index,
			style   = "bk-title-line"
		})
		line.style.width = display_width * 0.41
	end
end

function addTextToDescrptionGUIPane(pane, tab_name, text_name, localized, index, horizontal_align)
	if pane and tab_name then
		if text_name == nil then
			text_name = ""
		end
		if localized == nil then
			localized = true
		end
		local display_width = getScaledDisplayWidth(pane.player_index)
		-- Text flow
		local text_flow = booktorio.gui.addFlow(pane, 
		{
			name      = w_prefix..tab_name.."text-flow" .. index,
			direction = "horizontal",
			style     = "bk-text-flow"
		})
		text_flow.style.width = display_width * 0.82
		text_flow.style.horizontal_align = horizontal_align
		-- Description
		booktorio.gui.addDescription(text_flow,
		{
			name    = w_prefix..tab_name.."description-" .. index, 
			caption = getFormattedCaption(text_name, localized),
			style   = "bk-wiki-description-label"
		})		
	end
end

function addImageToDescrptionGUIPane(pane, tab_name, image_name, index, horizontal_align)
	if pane and tab_name and image_name then
		local display_width = getScaledDisplayWidth(pane.player_index)
		local image_flow = booktorio.gui.addFlow(pane, 
		{
			name      = w_prefix..tab_name.."image-flow-" .. index,
			direction = "horizontal",
			style     = "bk-wiki-image-flow"
		})
		image_flow.style.width = display_width * 0.82
		image_flow.style.horizontal_align = horizontal_align
		if game.is_valid_sprite_path(image_name) then
			local image = booktorio.gui.addSprite(image_flow,
			{
				name   = w_prefix..tab_name.."preview-" .. index, 
				sprite = image_name
			})
			image.resize_to_sprite = true 
		end
	end
end

function getPlayerDisplayResolution(player_index)
	local player = game.players[player_index]
	if player and player.valid then
		return player.display_resolution
	end
end

function getScaledDisplayWidth(player_index)
	local display_resolution = getPlayerDisplayResolution(player_index)
	if display_resolution then
		local scale = game.players[player_index].display_scale
		return display_resolution.width / scale
	end
	return 0
end

function getScaledDisplayHeight(player_index)
	local display_resolution = getPlayerDisplayResolution(player_index)
	if display_resolution then
		local scale = game.players[player_index].display_scale
		return display_resolution.height / scale
	end
	return 0
end

function changeWikiDescription(event, tab_name)
	local list = booktorio.gui.getElementByName(event.player_index, w_prefix..tab_name.."topics-list")
	if list then
		local topic = global.booktorio.threads[tab_name].topics[list.selected_index]
		local wiki_info_pane = getDescrptionGUIPane(event, tab_name)
		if topic and wiki_info_pane then -- if topic and wiki window exist
			local subtitle_count, texts_count, image_count = 0, 0, 0
			local element_type = nil
			local element_localized = nil
			local tilet_added_or_skipped = false
			local topic_align = global.booktorio.threads[tab_name].topics_align[list.selected_index] or "left"
			for _, element in pairs(topic) do
				element_type = element.type or element[1]
				
				if element_type == "title" and not tilet_added_or_skipped then
					-- Add title
					element_localized = true
					if element.localized ~= nil then
						element_localized = element.localized
					elseif element[3] ~= nil then
						element_localized = element[3]
					end
					addTitleToDescrptionGUIPane(wiki_info_pane, tab_name, element.title or element[2], element_localized, topic_align)
				elseif element_type == "subtitle" then
					-- Add subtitle
					subtitle_count = subtitle_count + 1
					element_localized = true
					if element.localized ~= nil then
						element_localized = element.localized
					elseif element[3] ~= nil then
						element_localized = element[3]
					end
					addSubtitleToDescrptionGUIPane(wiki_info_pane, tab_name, element.subtitle or element[2], element_localized, subtitle_count, topic_align)
				elseif element_type == "image" then
					-- Add image
					image_count = image_count + 1
					addImageToDescrptionGUIPane(wiki_info_pane, tab_name, element.spritename or element[2], image_count, topic_align)
				elseif element_type == "text" then
					-- Add text
					texts_count = texts_count + 1
					element_localized = true
					if element.localized ~= nil then
						element_localized = element.localized
					elseif element[3] ~= nil then
						element_localized = element[3]
					end
					addTextToDescrptionGUIPane(wiki_info_pane, tab_name, element.text or element[2], element_localized, texts_count, topic_align)
				end
				
				tilet_added_or_skipped = true
			end
			local display_height = getScaledDisplayHeight(wiki_info_pane.player_index)
		end		
	end
end

function createWiki(event)	
	local player            = game.players[event.player_index]
	local display_width     = getScaledDisplayWidth(event.player_index)
	local display_height    = getScaledDisplayHeight(event.player_index)
	local player_gui_center = player.gui.screen

	-- Window
	local invisible_frame = booktorio.gui.addFrame(player_gui_center, 
	{
		name        = w_prefix.."invisible-frame", 
		style       = "invisible_frame"		
	})
	invisible_frame.auto_center  = true
	invisible_frame.style.vertical_align   = "center"
	invisible_frame.style.horizontal_align = "center"
	local outer_frame = booktorio.gui.addFrame(invisible_frame, 
	{
		name        = w_prefix.."outer-frame", 
		style       = "outer_frame"		
	})
	outer_frame.style.vertical_align   = "center"
	outer_frame.style.horizontal_align = "center"
	local wiki_frame = booktorio.gui.addFrame(outer_frame, 
	{
		name        = w_prefix.."main-frame", 
		caption     = {"gui.bk-wiki-name"},
		direction   = "vertical", 
		style       = "bk-wiki-window"		
	})	
	wiki_frame.style.width  = display_width
	wiki_frame.style.height = display_height
	
	-- Main Window Container 
	local wiki_main_flow = booktorio.gui.addFlow(wiki_frame, 
	{
		name      = w_prefix.."main-flow",
		direction = "horizontal",
		style     = "bk-wiki-window-flow"
	})
	wiki_main_flow.style.width  = display_width  * 0.98
	wiki_main_flow.style.height = display_height * 0.88
	-- Main tab pane
	local wiki_tab_pane = booktorio.gui.addPane(wiki_main_flow, 
	{
		name      = w_prefix.."main-tab-pane",
		direction = "horizontal",
		style     = "bk-wiki-tabbed-pane"
	})
	wiki_tab_pane.style.width  = display_width  * 0.98
	wiki_tab_pane.style.height = display_height * 0.88
	-- Add threads
	if global.booktorio.threads and next(global.booktorio.threads) then
		for thread_name, thread_def in pairs(global.booktorio.threads) do
			-- Tab definition
			local tab_caption = getFormattedCaption(thread_name, thread_def.localized)
			local tab_name = nil
			if type(tab_caption) == "string" then
				tab_name = tab_caption
			else
				tab_name = tab_caption[1]
			end		
			-- Tab title element
			local tab = booktorio.gui.addTab(wiki_tab_pane, 
			{
				name    = tab_name,
				caption = tab_caption,
				style   = "tab"
			})
			
			-- Tab content
			local tab_flow = booktorio.gui.addFlow(wiki_tab_pane, 
			{
				name      = w_prefix..tab_name.."-flow",
				direction = "horizontal",
				style     = "bk-wiki-window-flow"
			})
			tab_flow.style.width  = display_width  * 0.975
			tab_flow.style.height = display_height * 0.84
			-- -- LEFT
			----------------------------------
			-- Left container
			local left_flow = booktorio.gui.addFlow(tab_flow, 
			{
				name      = w_prefix..tab_name.."left-flow",
				direction = "vertical",
				style     = "bk-wiki-left-window-flow"
			})	
			left_flow.style.width  = display_width  * 0.13
			left_flow.style.height = display_height * 0.82
			--Title
			booktorio.gui.addLabel(left_flow, 
			{
				name    = w_prefix..tab_name.."list-title", 
				caption = {"gui.bk-topic-list-title"},
				style   = "caption_label"
			})	
			booktorio.gui.addLine(left_flow, 
			{
				name    = w_prefix..tab_name.."title-line"
			})
			-- Topics list
			booktorio.gui.addList(left_flow, 
			{
				name    = w_prefix..tab_name.."topics-list", 
				items   = thread_def.topics_names
			})
			
			-- -- RIGHT
			----------------------------------
			-- Area where show topics
			local wiki_info_pane = booktorio.gui.addScrollPane(tab_flow, 
			{
				name                     = w_prefix..tab_name.."info-pane", 
				horizontal_scroll_policy = "never", 
				vertical_scroll_policy   = "auto-and-reserve-space",
				style                    = "bk-wiki-description-flow"
			})	
			wiki_info_pane.style.width  = display_width  * 0.83
			wiki_info_pane.style.height = display_height * 0.80
			
			booktorio.gui.addDescription(wiki_info_pane, 
			{
				name=w_prefix..tab_name.."info", 
				caption={"gui.bk-info-description"},
				style   = "bk-wiki-description-label"
			})
			
			--Add tab
			wiki_tab_pane.add_tab(tab, tab_flow)
		end
		wiki_tab_pane.selected_tab_index = 1
	end
	
	-- Back button section
	local back_button_flow = booktorio.gui.addFlow(wiki_frame, 
	{
		name      = w_prefix.."back-button-flow",
		direction = "horizontal",
		style     = "bk-wiki-back-button-flow"
	})
	local inner_back_button_flow = booktorio.gui.addFlow(back_button_flow, 
	{
		name      = w_prefix.."inner-back-button-flow",
		direction = "horizontal",
		style     = "horizontal_flow"
	})	
	-- Back button
	booktorio.gui.addBackButton(inner_back_button_flow, 
	{
		name    = w_prefix.."close", 
		caption = {"gui.bk-close-wiki"},
		style   = "bk-wiki-back-button"
	})
	--Filler
	booktorio.gui.addElement
	(
		back_button_flow,
		"empty-widget",
		w_prefix.."bottom-filler",
		nil,
		nil,
		{ style = "draggable_space_with_no_right_margin" }
	)
	
	-- Add window frame to main player opened wiki
	player.opened = wiki_frame
end

function toggleWiki(event)
	local wiki = booktorio.gui.getElementByName(event.player_index, w_prefix.."invisible-frame")
	if wiki and wiki.valid then
		wiki.destroy()
	else
		if game.players[event.player_index].opened and game.players[event.player_index].opened.valid then
			game.players[event.player_index].opened.destroy()
			game.players[event.player_index].opened = nil
		end
		createWiki(event)	
	end
end

function closeWiki(event)
	local wiki_invisible_frame = booktorio.gui.getElementByName(event.player_index, w_prefix.."invisible-frame")
	if wiki_invisible_frame and wiki_invisible_frame.valid then
		wiki_invisible_frame.destroy()	
	end
end

function initializeWiki(event)	
	-- add to gui the main open button
	local button_flow = mod_gui.get_button_flow(game.players[event.player_index])
	booktorio.gui.addSpriteButton(button_flow, 
	{
		name    = w_prefix.."toggle-wiki",
		tooltip = {"gui.bk-open-wiki-tooltip"},
		sprite  = "bk-open-gui",
		style   = "bk-wiki-icon-button"
	})
end

function onResolutionChanged(event)
	closeWiki(event)
end

function onConfigurationChanged(event)
	local button = nil
	for _, player in pairs(game.players) do
		button = booktorio.gui.getElementByName(player.index, w_prefix.."toggle-wiki")		
		if not button then
			event.player_index = player.index
			initializeWiki(event)
		end
	end
end

-- Callbacks
booktorio.gui.addClickElementEvent(w_prefix.."toggle-wiki", "toggleWiki")	
booktorio.gui.addClickElementEvent(w_prefix.."close", "closeWiki")	
local function onListClick(event)
	local tabbed_pane = booktorio.gui.getElementByName(event.player_index, w_prefix.."main-tab-pane")
	if tabbed_pane then
		local tab_name = tabbed_pane.tabs[tabbed_pane.selected_tab_index].tab.name
		if event.element.name == w_prefix..tab_name.."topics-list" then
			changeWikiDescription(event, tab_name)
		end
	end
end

return
{ 
	-- -- Bootstrap
	{ initializeWiki, "on_player_created" },
	{ onConfigurationChanged, "on_configuration_changed" },
	{ onResolutionChanged, "on_player_display_resolution_changed"},
	-- GUI actions
	{ closeWiki, "on_gui_closed" },	
	-- Special lists selection event
	{ onListClick, "on_gui_selection_state_changed"}
}
